%if style == newcode
\begin{code}
module BasicIO(rewrite, fileClass) where
import Control.Monad.State(liftIO)
import System.Directory(getDirectoryContents,doesDirectoryExist)

import VirtualMachine
import ObjectTheory
import Lists(storeNewValuesAsList)
\end{code}
%endif

%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\chapter{Basic IO primitives}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\begin{code}
rewrite                                       (State  heap  env  stack  ( FwApp  (Prim "stdout" "write") [a] )) = do
  liftIO (putStr (getStringValue heap a)) >>  (state  heap  env  stack  ( BwResult a_None                    ))
\end{code}

\begin{code}
rewrite  (State  heap  env  stack  ( FwApp  (Prim "os" "listdir") [a_x] )) = do
   contents <- liftIO (getDirectoryContents (getStringValue heap a_x))
   (state  ( storeNewValuesAsList a (map StringValue contents) heap )  env  stack  ( BwResult a ))
  where a = freeAddr heap
\end{code}

\begin{code}
rewrite  (State  heap  env  stack  ( FwApp  (Prim "path" "isdir") [a_x] )) = do
  isdir <- liftIO (doesDirectoryExist (getStringValue heap a_x))
  (state  heap  env  stack  ( FwExpr (BoolExpr isdir) ))
\end{code}

\begin{code}
rewrite  (State  heap  env  stack  ( FwApp  (Prim "file" "__new__") [a_c, a_x] )) = do
  contents <- liftIO (readFile (getStringValue heap a_x))
  let a = freeAddr heap
  (state  ( heap <+> [a |-> StringValue contents] )  env  stack  ( BwResult a ))
\end{code}




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%if style==newcode
\begin{code}
rewrite _ = skipState
\end{code}
%endif
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%if style==newcode
\begin{code}
fileClass :: Value
fileClass = new 
  [ __class__ |-> a_Type
  , __bases__ |-> a_objectT
  , __new__   |-> Prim "file" "__new__"
  , __call__  |-> Prim "file" "__new__"
  ]
\end{code}
%endif
%}
