\documentclass[fleqn]{beamer}

\usepackage{amsmath}
\usepackage{amsfonts}
\usepackage[silent]{polytable}
\usepackage{listings}
\usepackage{url}

\usepackage{color}
\usepackage{fix-cm}

\usepackage{tikz}
\usetikzlibrary{arrows,automata}

%\usetheme{Boadilla}
\usenavigationsymbolstemplate{} 
%\usecolortheme{seagull}
%\usefonttheme{serif}
%\useoutertheme{split}
%\useinnertheme{circles}
\useoutertheme{infolines}

%include polycode.fmt

\lstset{basicstyle=\footnotesize}

\newsavebox{\testS}
\newdimen\bracketHeight

\newcommand\Rule[3]{%
\mathhs
\savecolumns
\sbox{\testS}{$#1$}%
\restorecolumns
\sbox{\testS}{$#2$}%
\settoheight\bracketHeight{\usebox{\testS}}
\begin{equation*}
\begin{array}{@@{}l@@{}l@@{}}
&
\restorecolumns
#1
\\[1ex]
\restorecolumns
\Rightarrow\left\{\vphantom{\rule{0pt}{\bracketHeight}}\right.
&
%#2
\usebox{\testS}
\\
\multicolumn{2}{l}{
\restorecolumns
#3
}
\end{array}
\end{equation*}
\plainhs
}

\newcommand\SimpleRule[2]{%
\mathhs
\begin{equation*}
\begin{array}{@@{}l}
#1 \\
#2
\end{array}
\end{equation*}
\plainhs
}


%include Result.lhs
%include Attributes.lhs

%format length lst = "|"lst"|"
%format null lst = "|"lst"|" == 0

%format es = "\overline{e}"
%format as = "\overline{a}"
%format xs = "\overline{x}"

%format isObjValue (v) = v "\in \Omega"

%format head lst = lst"_1"
%format tail lst  = lst"_{2..}"

% mapping
%format rm m k = m "\ominus" k
%format <-> = "\ominus"
%format (get m (k)) = m"("k")"
%format `isin` = "\in"
%format `isnotin` = "\notin"
%format new lst = lst
%format empty = "\emptyset"
%format <+> = "\oplus"
%format |-> = "\rightarrow"

% formatting rules for exressions
%format VarExpr v = v
%format CallExpr (e) args = e(args)
%format BoolExpr (b) = b
%format IntExpr i = i
%format StringExpr s = s
%format AttrExpr e v = e "." v
%format ListExpr (es) = [es]
%format TupleExpr (es) = (es)
%format DictExpr (dict) = dict
%format SliceExpr x (y) = x"["y"]"
%format BinOpExpr op x y = x op y
%format UnaryOpExpr op x = op x
%format YieldExpr e = "\mathtt{yield}~" e
%format AddrExpr a = a

% formatting rules for statements
%format ExprStmt e = e
%format (AssStmt (v) (e)) = v "=" e
%format AttrAssStmt e_t v e = e_t "." v "=" e
%format SliceAssStmt e s v = e"["s"] = " v
%format DelStmt v = "\mathtt{del}~" v
%format AttrDelStmt e x = "\mathtt{del}~"e.x
%format SliceDelStmt x y = "\mathtt{del}~"x[y]
%format DefStmt v pars b = "\mathtt{def}~" v(pars) ":" b
%format RetStmt e = "\mathtt{return}~" e
%format PrintStmt e = "\mathtt{print}"(e)
%format PassStmt = "\mathtt{pass}"
%format IfStmt e bt bf = "\mathtt{if}~" e ":" bt "~\mathtt{else}:" bf
%format WhileStmt e b = "\mathtt{while}~" e ":" b
%format ForStmt x e b = "\mathtt{for}~" x "~\mathtt{in}~" e ":" b
%format ClassStmt (x) (e) (b) = "\mathtt{class}~" x"("e")" ":" b
%format TryStmt (bt) (e) (x) (be) = "\mathtt{try}:" bt "~\mathtt{except}~" e"," x ":" be
%format FinallyStmt (bt) (bf) = "\mathtt{try}:" bt "~\mathtt{finally}~:~" bf
%format RaiseStmt (e) = "\mathtt{raise}~" e 
%format ImportStmt n = "\mathtt{import}~" n
%format BreakStmt = "\mathtt{break}"
%format ContinueStmt = "\mathtt{continue}"
%format ExecStmt e env = "\mathtt{exec}~"e"~\mathtt{in}~"env

%format (ConsBlock (s) (b)) = s";~"b
%format (EmptyBlock) = "\{\}"

%format (appendStmt (s) (b)) = s";~"b

% states
%format (State (h) s1 (env) s2 (s) s3 (a) st) = "\bigl\langle" h s1 env s2 s s3 a st "\bigr\rangle"
%format (state (h) s1 (env) s2 (s) s3 (a) st) = "\bigl\langle" h s1 env s2 s s3 a st "\bigr\rangle"

%format SEP = ",\,"
%format STOP = "\vphantom{~}"
%format (CASE (x) (s) (c)) = s "\;\;" "\text{if }" c
%format (OTHE (x) (s) (c)) = s "\;\;" "\text{otherwise}"

% steps
%format FwBlock (b) = b
%format FwStmt (s) = s";"
%format FwExpr (e) = e
%format FwApp (r) (args) = r(args)
%format BwResult (r) = r
%format Raise (a) = "\mathtt{raise}~" a
%format Return (a) = "\mathtt{return}~" a
%format Yield (a) (stack) = "\mathtt{yield}~" a ",~" stack

% formatting rules for Values
%format FuncValue ps b e = "\lambda" ps ".~" b "\dashv" e
%format GenValue (e) (s) = "\mathcal{G.}" s " \dashv " e 
%format IntValue i = i
%format BoolValue b = b
%format (ObjValue (m) (p)) = (m, p)
%format StringValue s = s
%format ListValue (as) = [as]
%format TupleValue (as) = (as)
%format DictValue (d) = d
%format NoneValue = "\mathtt{None}"

%format . = "."
%format BlockFrame (b) = b
%format CallFrame        (es) = "\circ"(es)
%format ArgFrame a (as) (es) = a(as", \circ, "es)
%format FuncFrame p (as) = p(as)
%format ReturnMarker env = env "\vdash \circ_{\lambda}"
%format GenMarker env a =  env "\vdash \mathcal{G}."a
%format ReturnFrame = "\mathtt{return}~\circ"
%format AssFrame v = v "=" "\circ"
%format AttrAssFrame1 v e = "\circ." v "=" e
%format AttrAssFrame2 a v = a "." v "= \circ"
%format AttrDelFrame x = "\mathtt{del}~\circ."x
%format IfFrame1 bt bf = "\mathtt{if}~\circ :" bt "~\mathtt{else}:" bf
%format IfFrame2 bt bf = "\mathtt{if}~\circ".__nonzero__() " :" bt "~\mathtt{else}:" bf
%format WhileFrame1 e bw = "\mathtt{while}~\circ_{"e"} :" bw
%format WhileFrame2 e bw = "\mathtt{while}~\circ_{"e"}".__nonzero__()" :" bw
%format WhileMarker e b = "\mathtt{while}~"e": \circ_{"b"}"
%format ForFrame1 x b = "\mathtt{for}~"x"~\mathtt{in}~\circ :"b
%format ForFrame2 x b = "\mathtt{for}~"x"~\mathtt{in}~\circ".__iter__()" :"b
%format ForFrame3 x a b = "\mathtt{for}~"x"~\mathtt{in}~"a":"b
%format ForFrame4 x a b = "\mathtt{for}~"x"~\mathtt{in}~"a.__next__()":"b
%format AttrFrame v = "\circ." v
%format PrintFrame1 = "\mathtt{print}"("\circ".__str__())
%format PrintFrame2 = "\mathtt{print}(\circ)"
%format ClassFrame1 v b = "\mathtt{class}~" v "(\circ):" b
%format ClassFrame2 (x) (a) (env) = env "\vdash \mathtt{class}~" x (a)": \circ"
%format ClassConstrFrame x ax ab ac = "\marker_{"ac.__class__.__new__("\cdot", ax, ab, ac)"}"
%format ClassConstrMarker x ao = "\marker_{"ao.__class__.__init__("\cdots")"}"
%format RaiseFrame = "\mathtt{raise}~\circ"
%format ExceptMarker e x b = "\mathtt{except}~" e ",~" x ":" b
%format FinallyMarker b = "\mathtt{finally} :" b
%format RaiseContFrame a = "\marker^{\mathtt{raise}~"a"}_{\mathtt{finally}~:\circ}"
%format CatchingFrame a x b = "\marker^{\mathtt{raise}~"a"}_{\mathtt{except}~\circ,~"x":" b"}"
%format LeftBinOpFrame op e = "\circ" op e
%format RightBinOpFrame a op = a op "\circ"
%format LeftBoolOpFrame op e = "\circ" op"_{B}" e
%format UnaryOpFrame op = op "\circ"

%format ContSliceFrame k = "\circ"[k]
%format KeySliceFrame a = a["\circ"]
%format ValueSliceAssFrame c k = c[k] "= \circ"
%format ContSliceAssFrame k a = "\circ" [k] "=" a
%format KeySliceAssFrame c v = c ["\circ"] "=" v
%format ContSliceDelFrame k = "\mathtt{del}~\circ"[k]
%format KeySliceDelFrame a = "\mathtt{del}~"a["\circ"]

%format ExecFrame1 e = "\mathtt{exec}~\circ~\mathtt{in}~"e
%format ExecFrame2 a = "\mathtt{exec}~"a"~\mathtt{in}~\circ"
%format ExecMarker env = env "\vdash \mathtt{exec}~\circ"
%format ImportFrame (env) (xs) (ys) = env "\vdash \mathtt{import}~"xs."\!\circ\!".ys
%format YieldFrame = "\mathtt{yield}~\circ"
%format (ListFrame (as) (es)) = [as, "\circ", es]
%format (TupleFrame (as) (es)) = (as, "\circ", es)
%format (DictKeyFrame (d) (e) (eps)) = "\{"d, "\circ:" e, eps"\}"
%format (DictValFrame (d) (a) (eps)) = "\{"d, a ":\circ", eps"\}"
%format InstrFrame (i) = i

%format frame = "f"
%format stack = "S"

%format sSingleton (f) = f
%format EmptyStack = "\Box"
%format :|: = "|"
%format `sPrepend` = "|"
%format `sConcat` = "|"
%format `sElem` = "\in"

%format heap = "\Theta"
%format heap'
%format heap''

%format (getValue (a) heap) = heap"("a")"
%format freeAddr heap = "\text{new address}" "\notin" heap
%format (getEnvValue heap (a)) = heap(a)
%format (getIntValue (a) heap) = heap"("a")"
%format (getStringValue heap (a)) = heap(a)
%format (getBoolValue heap (a)) = heap(a)
%format getFuncValue heap (a) = heap(a)


%format (getFullEnv (envs) (heap)) = "\Sigma^{"heap"}_{"envs"}"
%format envs = "\Gamma"
%format envs'
%format envs''
%format env = "\gamma"
%format env_1
%format env_2
%format env_c
%format localVars heap result = heap(result)"_{loc}"

\newcommand\sectionframe[1]{%
\begin{frame}
\begin{center}
\LARGE{#1}
\end{center}
\end{frame}
}

\newcommand\humongous[1]{
\begin{center}
{\fontsize{60}{70}\selectfont\textcolor{gray}{#1}}
\end{center}
}

\title{An executable operational semantics for Python}
\author{Gideon Smeding}
\institute{Utrecht University}
\date{October 23, 2008}

\begin{document}
%%%%%%%%%%%%%%%%%%%%
\frame{\titlepage}
%%%%%%%%%%%%%%%%%%%%
\section*{Outline}
\begin{frame}
\tableofcontents
\end{frame}
%%%%%%%%%%%%%%%%%%%%
\section{Introduction}
%%%%%%%%%%%%%%%%%%%%
\begin{frame}[fragile]
\frametitle{An executable operational semantics for Python}

\begin{columns}

\begin{column}[T]{.5\textwidth}
\begin{block}{Formal semantics}
\begin{code}
rewrite  (State  heap  envs  stack  ( FwStmt PassStmt  )) =
         (state  heap  envs  stack  ( BwResult  a_None ))
\end{code}
\end{block}
\end{column}

\begin{column}[T]{.5\textwidth}
\begin{block}{Interpreter}
{\footnotesize
\begin{verbatim}
>>> def intpr(str) :
...     exec str in {}
... 
>>> intpr("print(\"Hello world!\")")
Hello world!
>>>
\end{verbatim}
}
\end{block}
\end{column}

\end{columns}

\end{frame}
%%%%%%%%%%%%%%%%%%%%
\subsection{Python}
%%%%%%%%%%%%%%%%%%%%
\begin{frame}
\frametitle{Python}

\begin{description}
\item[\textbf{What:}]
high level \\
general purpose \\
imperative \\
object-oriented \\
dynamic (interpreted) \\

\item[\textbf{Who:}] Guido van Rossum 

\item[\textbf{Where:}] CWI, now open source

\item[\textbf{When:}] 1991

%\item[\textbf{Why:}] 
\end{description}

\end{frame}
%%%%%%%%%%%%%%%%%%%%
%\begin{frame}
%\frametitle{Implementations}
%
%\begin{description}
%\item[CPython] the original
%\item[PyPy] python in python, with LLVM
%\item[IronPython] python for .net/mono
%\item[Jython] python for java
%\item[...] dialects, e.g., stackless Python and boo
%\end{description}
%
%\end{frame}
%%%%%%%%%%%%%%%%%%%%
\sectionframe{Demonstration}
%%%%%%%%%%%%%%%%%%%%
\begin{frame}[fragile]
\frametitle{Recap}

\lstset{language=Python}

\begin{columns}

\begin{column}[T]{.5\textwidth}

\begin{block}{Simple expressions}
\begin{lstlisting}
4 + 3 * 2
\end{lstlisting}
\end{block}

\begin{block}{Generator}
\begin{lstlisting}
def fib() :
    x = 1
    y = 1
    while True :
        yield x
        tmp = y  
        y = x + y
        x = tmp

f = fib()
print(f.next())
\end{lstlisting}
\end{block}

\end{column}


\begin{column}[T]{.5\textwidth}

\begin{block}{Functions}
\begin{lstlisting}
def fac(x) :
    if x <= 0 :
        return 1
    else :
        return fac(x-1) * x
\end{lstlisting}
\end{block}

\begin{block}{Classes}
\begin{lstlisting}
class Counter(object) :
    def __init__(self) :
        self.x = 1
    def inc(self) :
        self.x = x + 1

cnt = Counter()
cnt.inc()
print(cnt.x)
\end{lstlisting}
\end{block}

\end{column}

\end{columns}
\end{frame}
%%%%%%%%%%%%%%%%%%%%
\subsection{Formal semantics}
\sectionframe{Formal semantics}
%%%%%%%%%%%%%%%%%%%%
\begin{frame}
%\frametitle{What are formal semantics?}

\begin{quote}
a mathematical model describing the behaviour of a programming language
\end{quote}

\end{frame}
%%%%%%%%%%%%%%%%%%%%
\begin{frame}
\frametitle{Why formal semantics?}

\begin{columns}

\begin{column}[T]{.5\textwidth}
\textbf{Pro:} \\
\begin{itemize}
\item standardization
\item development of tools
\item language extensions
\item proofs
\end{itemize}
\end{column}

\begin{column}[T]{.5\textwidth}

\pause

\textbf{Con:} \\

\begin{itemize}
\item out of touch
\item inhibits experimentation
\end{itemize}
\end{column}

\end{columns}

\end{frame}
%%%%%%%%%%%%%%%%%%%%
\begin{frame}
\frametitle{An executable semantics}

\begin{columns}

\begin{column}[T]{.5\textwidth}
\textbf{Solution:} \\
\begin{itemize}
\item realism
\item enables experimentation
\end{itemize}
\end{column}

\begin{column}[T]{.5\textwidth}

\pause

\textbf{Bonus:} \\
\begin{itemize}
\item static analysis
\item testing
\item fun
\end{itemize}
\end{column}

\end{columns}


\end{frame}
%%%%%%%%%%%%%%%%%%%%
\begin{frame}
\frametitle{A formal semantics for Python}

Quite well documented
\vspace{1em}
\begin{itemize}
\item the reference manual
\item the CPython implementation
\item the PyPy implementation
\end{itemize}

\pause

\vspace{2em}
\textbf{But no formal semantics!}

\end{frame}
%%%%%%%%%%%%%%%%%%%%
\begin{frame}

\begin{quote}
``A lot of the operational behavior that people rely on basically comes from accidents of implementation[..]''
\end{quote}

\end{frame}
%%%%%%%%%%%%%%%%%%%%
\section{The semantic rules}
\sectionframe{The semantic rules}
\subsection{The abstract machine state}
%%%%%%%%%%%%%%%%%%%%
\begin{frame}
\frametitle{The abstract machine state}

\begin{center}
\Large{$\bigl\langle \Theta, \Gamma, S, \imath \bigr\rangle$}
\end{center}

\vspace{1cm}
\begin{center}
\begin{block}{where}

\begin{description}
\item[$\Theta$] is the heap
\item[$\Gamma$] is the environment-address stack
\item[$S$] is the call stack
\item[$\imath$] is the instruction
\end{description}

\end{block}
\end{center}

\end{frame}
%%%%%%%%%%%%%%%%%%%%
\begin{frame}
\frametitle{Heap}

\begin{columns}

\column{.4\textwidth}
\humongous{$\Theta$}

\column{.6\textwidth}

\textbf{The memory model}

\vspace{1em}
$\text{Address} \mapsto \text{Value}$

\pause

\begin{block}{}
%format a_1
%format a_2
%format a_3
%format a_4
\begin{code}
heap =  [ a_1  |->  42
        , a_2  |->  "spam"
        , a_3  |->  [a_1, a_2]
        , a_4  |->  ["x" |-> a_1]
        ]
\end{code}
\end{block}

\end{columns}

\end{frame}
%%%%%%%%%%%%%%%%%%%%%
\begin{frame}
\frametitle{Environment}

\begin{columns}

\column{.4\textwidth}
\humongous{$\Gamma$}

\column{.6\textwidth}

\textbf{A stack of scopes}

\vspace{1em}
a of addresses pointing to bindings

\pause

\begin{block}{}
\begin{code}
envs = EmptyStack :|: env_0 :|: env
\end{code}

\begin{code}
heap =  [ a_1    |->  42
        , env_0  |->  ["x" |-> a_1]
        ]
\end{code}
\end{block}

\end{columns}

\end{frame}
%%%%%%%%%%%%%%%%%%%%%
\begin{frame}
\frametitle{Call stack}

\begin{columns}

\column{.4\textwidth}
\humongous{$S$}

\column{.6\textwidth}

\textbf{A continuation stack}

\vspace{1em}
a stack of continuation frames

\pause

\begin{block}{}
\begin{code}
EmptyStack :|: AssFrame x
\end{code}
\end{block}

\end{columns}

\end{frame}
%%%%%%%%%%%%%%%%%%%%%
\begin{frame}
\frametitle{Instruction}

\begin{columns}

\column{.4\textwidth}
\humongous{$\iota$}

\column{.6\textwidth}

\textbf{The instruction pointer}

\vspace{1em}
the current instruction

\pause

\begin{block}{}
\begin{itemize}
\item blocks, statements, and expressions
\item a result address
\item an unwind instruction, e.g. return
\end{itemize}
\end{block}

\end{columns}

\end{frame}
%%%%%%%%%%%%%%%%%%%%
\subsection{Rewrite rules}
%%%%%%%%%%%%%%%%%%%%
\begin{frame}[fragile]
\frametitle{Rewriting the state machine}

\begin{center}
\Large{
$\bigl\langle \Theta, \Gamma, S, \imath \bigr\rangle
\Rightarrow
\bigl\langle \Theta', \Gamma', S', \imath' \bigr\rangle$
}
\end{center}

\pause

\vspace{2em}

\begin{block}{Assignments}

\savecolumns
\begin{code}
rewrite  (State  ( heap                        )  ( envs           )  ( stack                )  ( FwStmt (AssStmt x e) )) =
         (state  ( heap                        )  ( envs           )  ( stack :|: AssFrame x )  ( FwExpr e             ))
\end{code}

\pause

\vspace{-2em}

\restorecolumns
\begin{code}
rewrite  (State  ( heap                        )  ( envs :|: env_1 )  ( stack :|: AssFrame x )  ( BwResult a           )) =
         (state  ( heap'                       )  ( envs :|: env_1 )  ( stack                )  ( BwResult a_None      ))
  where heap' = heap <+> [env_1 |-> get heap env_1 <+> [x |-> a]]
\end{code}

%\restorecolumns
%\begin{code}
%rewrite  (State  ( heap                        )  ( envs           )  ( stack                )  ( FwExpr (IntExpr i)   )) =
%         (state  ( heap <+> [a |-> IntValue i] )  ( envs           )  ( stack                )  ( BwResult a           ))
%   where  a      = freeAddr heap
%\end{code}


%%format IfFrame2 b_if b_else = "\mathtt{if~\circ:~}" b_if "\mathtt{~else~:~}" b_else
%%format raise error args = "\mathtt{raise}~" error()
%%format b_if
%%format b_else
%\begin{code}
%rewrite                     (State  heap  envs  ( stack :|: IfFrame2 b_if b_else )  ( BwResult a           ))
%  | getBoolValue heap a  =  (state  heap  envs  ( stack                          )  ( FwBlock b_if         ))
%  | otherwise            =  (state  heap  envs  ( stack                          )  ( FwBlock b_else       ))
%\end{code}

\end{block}

\end{frame}
%%%%%%%%%%%%%%%%%%%%
\sectionframe{Tracing an example}
%%%%%%%%%%%%%%%%%%%%
\begin{frame}[fragile]
\frametitle{A simple program}

\begin{lstlisting}[xleftmargin=10em]
def const(x, y) :
    return y

const(1,2)
\end{lstlisting}

\end{frame}
%%%%%%%%%%%%%%%%%%%%
\begin{frame}[plain,shrink=20]
\frametitle{Tracing the execution}

%format uncover2 (a) = "\uncover<2->{"a"}"
%format uncover3 (a) = "\uncover<3->{"a"}"
%format uncover4 (a) = "\uncover<4->{"a"}"
%format uncover5 (a) = "\uncover<5->{"a"}"
%format uncover6 (a) = "\uncover<6->{"a"}"
%format uncover7 (a) = "\uncover<7->{"a"}"
%format uncover8 (a) = "\uncover<8->{"a"}"
%format uncover9 (a) = "\uncover<9->{"a"}"
%format uncover10 (a) = "\uncover<10->{"a"}"
%format uncover11 (a) = "\uncover<11->{"a"}"
%format uncover12 (a) = "\uncover<12->{"a"}"
%format uncover13 (a) = "\uncover<13->{"a"}"
%format uncover14 (a) = "\uncover<14->{"a"}"
%format uncover15 (a) = "\uncover<15->{"a"}"
%format uncover16 (a) = "\uncover<16->{"a"}"

%format heap_0
%format heap_1
%format heap_2
%format heap_3
%format heap_4
%format env_0
%format a_1
%format a_2
%format lambda = "\lambda"
%format a_lambda
%format env_lambda
%format def = "\mathtt{def}"
%format return = "\mathtt{return}"
%format circ = "\circ"
\begin{code}
                (state  heap_0  ( env_0                        )  ( EmptyStack                                                  )  ( def const(x,y):...; const(1,2) ))
uncover2   (=>  (state  heap_0  ( env_0 :|: env                )  ( EmptyStack :|: const(1, 2)                                  )  ( def const(x,y):...             )))
uncover3   (=>  (state  heap_1  ( env_0 :|: env                )  ( EmptyStack :|: const(1, 2)                                  )  ( a_None                         )))
uncover4   (=>  (state  heap_1  ( env_0 :|: env                )  ( EmptyStack                                                  )  ( const(1, 2)                    )))
uncover5   (=>  (state  heap_1  ( env_0 :|: env                )  ( EmptyStack :|: CallFrame (1, 2)                             )  ( const                          )))
uncover6   (=>  (state  heap_1  ( env_0 :|: env                )  ( EmptyStack :|: CallFrame (1, 2)                             )  ( a_lambda                       )))
uncover7   (=>  (state  heap_1  ( env_0 :|: env                )  ( EmptyStack :|: a_lambda(circ, 2)                            )  ( 1                              )))
uncover8   (=>  (state  heap_2  ( env_0 :|: env                )  ( EmptyStack :|: a_lambda(circ, 2)                            )  ( a_1                            )))
uncover9   (=>  (state  heap_2  ( env_0 :|: env                )  ( EmptyStack :|: a_lambda(a_1, circ)                          )  ( 2                              )))
uncover10  (=>  (state  heap_3  ( env_0 :|: env                )  ( EmptyStack :|: a_lambda(a_1, circ)                          )  ( a_2                            )))
uncover11  (=>  (state  heap_3  ( env_0 :|: env                )  ( EmptyStack                                                  )  ( a_lambda(a_1, a_2)             )))
uncover12  (=>  (state  heap_4  ( env_0 :|: env :|: env_lambda )  ( EmptyStack :|: ReturnMarker (env_0 :|: env)                 )  ( return y                       )))
uncover13  (=>  (state  heap_4  ( env_0 :|: env :|: env_lambda )  ( EmptyStack :|: ReturnMarker (env_0 :|: env) :|: ReturnFrame )  ( y                              )))
uncover14  (=>  (state  heap_4  ( env_0 :|: env :|: env_lambda )  ( EmptyStack :|: ReturnMarker (env_0 :|: env) :|: ReturnFrame )  ( a_2                            )))
uncover15  (=>  (state  heap_4  ( env_0 :|: env :|: env_lambda )  ( EmptyStack :|: ReturnMarker (env_0 :|: env)                 )  ( return a_2                     )))
uncover16  (=>  (state  heap_4  ( env_0 :|: env                )  ( EmptyStack                                                  )  ( a_2                            )))
\end{code}

%format (withenv (env)) = "\dashv" env
\only<3>{
\begin{code}
heap_1 = heap_0  <+>  [ a_lambda  |-> \x y . return y (withenv (env_0 :|: env)) 
                      , env       |-> get heap env <+> ["const" |-> a_lambda] ]
\end{code}
}

\only<8>{
\begin{code}
heap_2 = heap_1  <+>  [a_1 |-> IntValue 1]
\end{code} }

\only<10>{
\begin{code}
heap_3 = heap_2  <+>  [a_2 |-> IntValue 2]
\end{code}}

\only<12>{
\begin{code}
heap_4 = heap_2  <+>  [ env_lambda |->  [ "x" |-> a_1 
                                        , "y" |-> a_2 ]]
\end{code}}

\end{frame}
%%%%%%%%%%%%%%%%%%%%
\section{The realization}
\subsection{Literate programming}
\sectionframe{Literate programming}
%%%%%%%%%%%%%%%%%%%%
\begin{frame}

\begin{quote}
``The main idea is to regard a program \\
as a communication to human beings rather than\\
as a set of instructions to a computer.'' \\
-Knuth
\end{quote}

\end{frame}

\begin{frame}

\begin{quote}
``The main idea is to regard \textbf{a semantics}\\
as a communication to human beings \textbf{as well}\\
as a set of instructions to a computer.'' \\
-me
\end{quote}

\end{frame}
%%%%%%%%%%%%%%%%%%%%
\begin{frame}
\frametitle{Two targets}

\begin{center}
\begin{tikzpicture}[->,>=stealth',shorten >=1pt,auto,node distance=2.8cm,
                    semithick]

  \node[state] (A)                    {lhs};
  \node[state] (B) [below left  of=A] {pdf};
  \node[state] (C) [below right of=A] {exe};

  \path (A) edge [bend right] node {} (B)
        (A) edge [bend left ] node {}      (C);
\end{tikzpicture}
\end{center}

%\begin{tikzpicture}[->,>=stealth',shorten >=1pt,auto,node distance=2.8cm,
%                    semithick]
%
%  \node[state] (A)                    {lhs};
%  \node[state] (B) [above right of=A] {phs};
%  \node[state] (C) [below right of=A] {exe};
%  \node[state] (D) [right of=B]       {tex};
%  \node[state] (E) [right of=D]       {pdf};
%
%  \path (A) edge [bend left ] node {convert.pl} (B)
%        (A) edge [bend right] node {ghc}      (C)
%        (B) edge              node {lhs2\TeX} (D)
%        (D) edge              node {\LaTeX}    (E);
%\end{tikzpicture}

\end{frame}
%%%%%%%%%%%%%%%%%%%%
\begin{frame}[fragile]
\frametitle{Formatting a rule}

\lstset{language=Haskell}
\include{Defence-example}

\pause

\begin{code}
rewrite                   (State  heap  envs   ( stack :|: frame )  ( Return a   )) = case frame of
  ReturnMarker envs'  ->  (state  heap  envs'  ( stack           )  ( BwResult a ))
  otherwise           ->  (state  heap  envs   ( stack           )  ( Return a   ))
\end{code}

\end{frame}
%%%%%%%%%%%%%%%%%%%%
\subsection{Testing}
\sectionframe{Testing}
%%%%%%%%%%%%%%%%%%%
\begin{frame}
\frametitle{A test suite}

\begin{columns}

\column{.5\textwidth}
\begin{itemize}
\item existing regression tests?
\item reverse engineering tests
\item tests the formal semantics!
\item subset of Python
\end{itemize}

\pause

\column{.5\textwidth}
\begin{tabular}{lr}
\textbf{implementation} & \textbf{failed} \\
\hline
\hline
CPython        & 1   \\
PyPy           & 4   \\
Iron Python    & 13  \\
minpy          & 13  \\
Jython         & 15  \\
\hline
of             & 138    
\end{tabular}

\end{columns}

\end{frame}
%%%%%%%%%%%%%%%%%%%%
\section{Conclusion}
\sectionframe{Conclusion}
%%%%%%%%%%%%%%%%%%%%
\begin{frame}
\frametitle{What has been done}

\begin{columns}

\begin{column}[T]{.5\textwidth}
\textbf{What has been done}

\begin{itemize}
\item all major constructs
\begin{itemize}
\item functions and generators
\item objects and classes
\item operators
\item exceptions
\item if, while, and for
\item exec and import
\end{itemize}
\item all major types
\end{itemize}

\end{column}

\pause

\begin{column}[T]{.5\textwidth}
\textbf{What has not been done}
\begin{itemize}
\item no syntactic sugar
\begin{itemize}
\item $a~+=~1$
\item $[x~\mathtt{for}~x~\mathtt{in}~lst]$
\item else-if statements
\end{itemize}
\item no global, with
\item no reflection
\end{itemize}

%\vspace{2em}
%\large{Can be added easily}
\end{column}

\end{columns}

\pause

\begin{center}
\large{A significant subset of Python}
\end{center}

\end{frame}
%%%%%%%%%%%%%%%%%%%%%
\begin{frame}
\frametitle{Future work}

\begin{columns}

\column{.5\textwidth}
\textbf{Continued research}
\begin{itemize}
\item garbage collection
\item concurrency
\item foreign functions
\end{itemize}

\pause 

\column{.5\textwidth}
\textbf{New research}
\begin{itemize}
\item analyses
\item extensions
\item proofs
\end{itemize}

\end{columns}

\end{frame}
%%%%%%%%%%%%%%%%%%%%
\sectionframe{The end}
%%%%%%%%%%%%%%%%%%%%
\end{document}
