%if style == newcode
\begin{code}
module Dictionaries(o_dict, rewrite) where
import VirtualMachine
import ObjectTheory

o_dict :: Value
\end{code}
%endif
%format o_dict

%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Dictionaries}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%\begin{itemize}
%\item totally  mutable
%\item is mapping of strings to values
%\end{itemize}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection*{Dictionary class}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%format a_d
\begin{code}
o_dict = new
  [ __class__     |->  a_Type
  , __bases__     |->  a_objectT
  , __getitem__   |->  Typed (Prim "dict" "__getitem__") [a_dict, a_str]
  , __setitem__   |->  Typed (Prim "dict" "__setitem__") [a_dict, a_str, a_object]
  , __delitem__   |->  Typed (Prim "dict" "__delitem__") [a_dict, a_str]
  , __new__       |->  Prim "dict" "__new__"
  ]
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection*{Dictionary literals}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


\rulesection{Evaluating a dictionary literal}
\label{fw-rewrite-DictExpr}
%format firstKeyFrame e pes = "\{\circ:" e, pes"\}"
%format ess = "\overline{"es"}"
%format headKey ess = ess"_{1;1}"
%format headVal ess = ess"_{1;2}"
%format a_k
%format a_v
%format e_v
\begin{code}
rewrite               (State  ( heap                             )  env  ( stack                                                 )  ( FwExpr (DictExpr ess) ))
  | length ess > 0 =  (state  ( heap                             )  env  ( stack :|: DictKeyFrame empty (headVal ess) (tail ess) )  ( FwExpr (headKey ess)  ))
  | otherwise      =  (state  ( heap <+> [a |-> DictValue empty] )  env  ( stack                                                 )  ( BwResult a            ))
  where a = freeAddr heap
\end{code}


\rulesection{Evaluating the value of a key-value pair}
\begin{code}
rewrite (State  heap  env  ( stack :|: DictKeyFrame d e_v ess )  ( BwResult a_k )) =
        (state  heap  env  ( stack :|: DictValFrame d a_k ess )  ( FwExpr e_v   ))
\end{code}


\rulesection{Evaluating key-value pairs in a dictionary literal}
\begin{code}
rewrite                  (State  ( heap                                             )  env  ( stack :|: DictValFrame d a_k ess                                       )  ( BwResult a_v         ))
    | length ess > 0  =  (state  ( heap                                             )  env  ( stack :|: DictKeyFrame (d <+> [(k |-> a_v)])  (headVal ess) (tail ess) )  ( FwExpr (headKey ess) ))
    | otherwise       =  (state  ( heap <+> [ a |-> DictValue (d <+> [k |-> a_v]) ] )  env  ( stack                                                                  )  ( BwResult a           ))
  where  a  = freeAddr heap
         k  = getStringValue heap a_k
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection*{Dictionary primitives}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\rulesection{The dictionary constructor primitive}
\begin{code}
rewrite  (State  ( heap                                                                  )  env  stack  ( FwApp (Prim "dict" "__new__") [a_c] )) =
         (state  ( heap <+> [a |-> ObjValue (new [__class__ |-> a_c]) (DictValue empty)] )  env  stack  ( BwResult a                          ))
  where a =  freeAddr heap
\end{code}

\rulesection{The dictionary constructor primitive with starting dictionary}
\begin{code}
rewrite  (State  ( heap                                                                                    )  env  stack  ( FwApp (Prim "dict" "__new__") [a_c,a_d] )) =
         (state  ( heap <+> [a |-> ObjValue (new [__class__ |-> a_c]) (DictValue (getDictValue heap a_d))] )  env  stack  ( BwResult a                              ))
  where a =  freeAddr heap
\end{code}

\rulesection{The set item with key primitive}
\begin{code}
rewrite   (State  ( heap                     )  env  stack  ( FwApp (Prim "dict" "__setitem__") [a_self, a_i, a_v] )) =
          (state  ( heap <+> [a_self |-> d'] )  env  stack  ( BwResult a_None                                      ))
  where  k   = getStringValue heap a_i
         d'  = ObjValue (getObj heap a_self) (DictValue (getDictValue heap a_self <+> [k |-> a_v]))
\end{code}


\rulesection{The get item with key primitive}
\begin{code}
rewrite                                   (State  heap  env  stack  ( FwApp (Prim "dict" "__getitem__") [a_self, a_i] ))
   | i `isin` getDictValue heap a_self  =  (state  heap  env  stack  ( BwResult (get (getDictValue heap a_self) i)    ))
   | otherwise                          =  (state  heap  env  stack  ( Raise a_IndexError                             ))
  where i = getStringValue heap a_i
\end{code}


\rulesection{The delete item with key primitive}
\begin{code}
rewrite   (State  ( heap                     )  env  stack  ( FwApp (Prim "dict" "__delitem__") [a_self, a_i] )) =
          (state  ( heap <+> [a_self |-> d'] )  env  stack  ( BwResult a_None                                 ))
  where  k   = getStringValue heap a_i
         d'  = ObjValue (getObj heap a_self) (DictValue (getDictValue heap a_self <-> k))
\end{code}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%if style==newcode
\begin{code}
rewrite _ = skipState
\end{code}
%endif
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%}
%if style==newcode
\begin{code}
emptyDictValue = DictValue empty
firstKeyFrame e pes = DictKeyFrame empty e pes


headKey = fst . head
headVal = snd . head
\end{code}
%endif
