%if style == newcode
\begin{code}
module Exec where
import VirtualMachine
import ObjectTheory
import PythonParser
\end{code}
%endif

%{
%format e_p
%format a_p
%format e_env
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{The exec statement} \label{section:exec}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
An exec statement~|ExecStmt e_p e_env| executes a program~|e_p| in the environment~|e_env|. The program~|e_p| should be a string value and the environment~|e_env| a dictionary.

\begin{lstlisting} [ caption={Executing an important question}
                   , label=example:exec]
exec "v * 2" in { "v" : 21 }
\end{lstlisting}

\rulesection{Evaluating the string to be executed}
\label{fw-rewrite-ExecStmt}
First, the program string~|e_p| is evaluated.

\begin{code}
rewrite (State  heap  envs  ( stack                      )  ( FwStmt (ExecStmt e_p e_env) )) =
        (state  heap  envs  ( stack :|: ExecFrame1 e_env )  ( FwExpr e_p                  ))
\end{code}


\rulesection{Evaluating the execution environment}
\label{bw-rewrite-ExecFrame1}
Then, the environment dictionary~|e_env| is evaluated.

\begin{code}
rewrite (State  heap  envs  ( stack :|: ExecFrame1 e_env )  ( BwResult a_p )) =
        (state  heap  envs  ( stack :|: ExecFrame2 a_p   )  ( FwExpr e_env ))
\end{code}

%Note that exec statements must be valid blocks. This ensures that locality for break, continue and returns statements is maintained.


\rulesection{Execute the string}
\label{bw-rewrite-ExecFrame2}
\begin{enumerate}
\item If |a_p| is a string and |env| is a dictionary, we store the current environment on the stack, set the new environment and parse the program. The new environment is defined as the default environment~|env_0| extended with the dictionary environment~|env|.
\item If either |a_p| is not a block, or |env| is not a dictionary, we raise a |TypeError|.
\end{enumerate}

%{
%format parse str = "\text{parse}~" str
%format isInstOf a b c = a ":_"heap c
\begin{code}
rewrite                                                     (State  heap  ( envs                     )  ( stack :|: ExecFrame2 a_p  )  ( BwResult env                    ))
  | isInstOf a_p a_str heap && isInstOf env a_dict heap  =  (state  heap  ( sSingleton env_0 :|: env )  ( stack :|: ExecMarker envs )  ( parse (getStringValue heap a_p) ))
  | otherwise                                            =  (state  heap  ( envs                     )  ( stack                     )  ( raise a_TypeError []            ))
\end{code}
%}

The parse function will, if the string has the correct grammar, return a block. If the parser fails however, it wil raise a |SyntaxError| exception.


\rulesection{Restoring the old environment}
\label{bw-rewrite-ExecMarker}
Once the program string has been executed, the old environment |envs'| is reinstated and we return |None|.

\begin{code}
rewrite (State  heap  envs   ( stack :|: ExecMarker envs' )  ( BwResult a      )) =
        (state  heap  envs'  ( stack                      )  ( BwResult a_None ))
\end{code}



\rulesection{Raising an exception in an execution environment}
\label{raise-rewrite-ExecMarker}
The old environment is also reinstated when an exception is raised during the execution of the program (see Chapter~\ref{chapter:exceptions}). Return, yield, break and continue statements however, can not escape the execution environment, because they cannot occur outside of their respective compound statements.

\begin{code}
rewrite  (State  heap  envs  ( stack :|: ExecMarker envs' )  ( Raise a )) =
         (state  heap  envs' ( stack                      )  ( Raise a ))
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%if style==newcode
\begin{code}
rewrite _ = skipState
\end{code}
%endif
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%if style==newcode
\begin{code}
parse str = 
  case strictParseString pBlock str of
    Just b -> FwBlock b
    Nothing -> raise a_SyntaxError []
\end{code}
%endif
%}

