%if style == newcode
\begin{code}
module Lists(rewrite, storeNewValuesAsList, listIterType, o_list) where
import VirtualMachine
import ObjectTheory

o_list :: Value
\end{code}
%endif

%format o_list

%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Lists}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%\begin{itemize}
%\item totally mutable
%\item can be derived from
%\end{itemize}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection*{The list class}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%format ListIterator = "\mathtt{ListIterator}"
\begin{code}
o_list = new
  [ __class__    |->  a_Type
  , __bases__    |->  a_objectT
  , __delitem__  |->  Typed (Prim "list" "__delitem__") [a_list, a_int]
  , __getitem__  |->  Typed (Prim "list" "__getitem__") [a_list, a_int]
  , __setitem__  |->  Typed (Prim "list" "__setitem__") [a_list, a_int]
  , __new__      |->  Prim "list" "__new__"
  , __add__      |->  Prim "list" "__add__"
  , __len__      |->  Prim "list" "__len__"
  , __iter__     |->  Prim "list" "__iter__"
  , append       |->  Prim "list" "append"
  ]
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection*{Literal strings}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%List listerals are lists of expressions between brackets e.g. $[1,f(3),"a"]$. The expressions are evaluated one by one, from left to right.


\rulesection{Evaluating a list literal}
\label{fw-rewrite-ListExpr}
%If a list literal is not empty, we evaluate the first element and put the remaining elements on the stack. Otherwise, if the list is empty, a new empty list is created on the heap and its address is returned.

%format firstListFrame (es) = ["\circ", es]
%format emptyListValue = []
\begin{code}
rewrite          (State  ( heap                             )  env  ( stack                        )  ( FwExpr (ListExpr es) )) = case es of
  (e:es')    ->  (state  ( heap                             )  env  ( stack :|: firstListFrame es' )  ( FwExpr e             ))
  otherwise  ->  (state  ( heap <+> [a |-> emptyListValue ] )  env  ( stack                        )  ( BwResult a           ))
   where a = freeAddr heap
\end{code}


\rulesection{Evaluating the elements of a list literal}
%When evaluation of a list element completes, and not all elements have been evaluated yet, the evaluated element |a| is added to the stack and the next element is evaluated. Otherwise, if all elements have been evaluated, the list of elements is stored on the heap and its address is returned.

\begin{code}
rewrite            (State  ( heap                                    )  env  ( stack :|: ListFrame as es           )  ( BwResult a  )) = case es of
    (e:es')    ->  (state  ( heap                                    )  env  ( stack :|: ListFrame (as ++ [a]) es' )  ( FwExpr e    ))
    otherwise  ->  (state  ( heap <+> [a' |-> ListValue (as ++ [a])] )  env  ( stack                               )  ( BwResult a' ))
  where a' = freeAddr heap
\end{code}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection*{List primitives}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\rulesection{The list constructor primitive}
\begin{code}
rewrite  (State  ( heap               )  env  stack  ( FwApp (Prim "list" "__new__") [a_c] )) =
         (state  ( heap <+> [a |-> o] )  env  stack  ( BwResult a                          ))
   where  a =  freeAddr heap
          o =  ObjValue (new [__class__ |-> a_c]) emptyListValue
\end{code}


\rulesection{The length of list primitive}
\begin{code}
rewrite  (State  heap  env  stack  ( FwApp (Prim "list" "__len__") [a]               )) =
         (state  heap  env  stack  ( FwExpr (IntExpr (length (getListValue heap a))) ))
\end{code}


\rulesection{The get item at index primitive}
\begin{code}
rewrite               (State  heap  env  stack  ( FwApp (Prim "list" "__getitem__") [a_self, a_i] ))
   | i < length as =  (state  heap  env  stack  ( BwResult (as!!i)                                ))
   | otherwise     =  (state  heap  env  stack  ( raise a_IndexError []                           ))
  where  i   = getIntValue heap a_i
         as  = getListValue heap a_self
\end{code}


\rulesection{The set item at index primitive}
%{
%format setElem i a as = "\cdots", "a_{i-1}", a, "a_{i+1}", "\cdots"
\begin{code}
rewrite                (State  ( heap                      )  env  stack  ( FwApp (Prim "list" "__setitem__") [a_self, a_i, a] ))
   | i < length as  =  (state  ( heap <+> [a_self |-> as'] )  env  stack  ( BwResult a_None                                    ))
   | otherwise      =  (state  ( heap                      )  env  stack  ( raise a_IndexError []                              ))
  where  i    = getIntValue heap a_i
         as   = getListValue heap a_self
         as'  = ListValue (setElem i a as)
\end{code}
%}


\rulesection{The delete item at index primitive}
%format delElem i as = "\cdots", "a_{i-1}", "a_{i+1}", "\cdots"
\begin{code}
rewrite               (State  ( heap                      )  env  stack  ( FwApp (Prim "list" "__delitem__") [a_self, a_i] ))
   | i < length as =  (state  ( heap <+> [a_self |-> as'] )  env  stack  ( BwResult a_None                                 ))
   | otherwise     =  (state  ( heap                      )  env  stack  ( raise a_IndexError []                           ))
  where  i    = getIntValue heap a_i
         as   = getListValue heap a_self
         as'  = ListValue (delElem i as)
\end{code}


\rulesection{The append to list primitive}
\begin{code}
rewrite  (State  ( heap                     )  env  stack  ( FwApp (Prim "list" "append") [a_self, a] )) =
         (state  ( heap <+> [a_self |-> as] )  env  stack  ( BwResult a_None                          ))
  where as = ListValue (getListValue heap a_self ++ [a])
\end{code}


\rulesection{The list addition operator primitive}
%{
\begin{code}
rewrite   (State  ( heap                )  env  stack  ( FwApp (Prim "list" "__add__") [a_self, a_other] )) =
          (state  ( heap <+> [a |-> as] )  env  stack  ( BwResult a                                      ))
  where  as  = ListValue (getListValue heap a_self ++ getListValue heap a_other)
         a   = freeAddr heap
\end{code}
%}

%if style==newcode
\rulesection{Create a list iterator for a list}
\begin{code}
rewrite  (State  ( heap                                  )  env  stack  ( FwApp (Prim "list" "__iter__") [a_self] )) =
         (state  ( heap <+> [a |-> ListItValue a_self 0] )  env  stack  ( BwResult a                              ))
  where a = freeAddr heap
\end{code}


\rulesection{The list iterator is also an iterable}
\begin{code}
rewrite  (State  heap  env  stack  ( FwApp (Prim "ListIterator" "__iter__") [a_self] )) =
         (state  heap  env  stack  ( BwResult a_self                                 ))
\end{code}


\rulesection{Getting the next item of the list iterator}
%format a_l
\begin{code}
rewrite                 (State  ( heap                                                 )  env  stack  ( FwApp (Prim "ListIterator" "__next__") [a_self] ))
   | i < length as   =  (state  ( heap <+> [a_self |-> ListItValue a_l (i+1)]          )  env  stack  ( BwResult (as!!i)                                ))
   | otherwise       =  (state  ( heap <+> [a_self |-> GenValue EmptyStack EmptyStack] )  env  stack  ( raise a_StopIteration []                        ))
  where  (a_l, i)  = getListItValue heap a_self
         as        = getListValue heap a_l
\end{code}
%endif

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%if style==newcode
\begin{code}
rewrite _ = skipState
\end{code}
%endif
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%}
%if style==newcode
\begin{code}
emptyListValue = ListValue []
firstListFrame es = ListFrame [] es

listIterType :: Value
listIterType = new
  [ __class__ |-> a_Type
  , __bases__ |->  a_objectT
  , __next__  |-> Prim "ListIterator" "__next__"
  , __iter__  |-> Prim "list" "__iter__" --TODO: should return itself, not a new iterator
  ]

-- pre-condition: the index is smaller than the list's length
setElem :: Int -> a -> [a] -> [a]
setElem 0 e (_:tl) = e : tl
setElem i e (hd:tl) = hd : setElem (i-1) e tl

-- pre-condition: the index is smaller than the list's length
delElem :: Int -> [a] -> [a]
delElem 0 (_:tl) = tl
delElem i (hd:tl) = hd : delElem (i-1) tl

storeNewValuesAsList :: Result -> [Value] -> Heap -> Heap
storeNewValuesAsList listAddr vs heap =
  let (vsAddrs, heapWithValues) = foldr storeArg ([], heapWithList) vs
      storeArg v (as, h) = (a:as, h <+> [a |-> v])
        where a = freeAddr h
      heapWithList = heap <+> [listAddr |-> ListValue vsAddrs]
  in heapWithValues
\end{code}
%endif
