%if style == newcode
\begin{code}
module Numbers(rewrite, o_int) where
import VirtualMachine
import ObjectTheory

o_int :: Value
\end{code}
%endif

%format o_int

%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Integer values}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%\begin{itemize}
%\item integers are immutable
%\item can be inherited by user defined classes
%\item doesn't coerce
%\item implements all number primitives
%\end{itemize}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection*{The integer class} \label{section:integer-class}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\begin{code}
o_int = new  
  [ __class__     |-> a_Type
  , __bases__     |-> a_objectT
  , __new__       |-> Prim "int" "__new__"
  , __add__       |-> Typed (Prim "int" "__add__") [a_int, a_int]
  , __sub__       |-> Typed (Prim "int" "__sub__") [a_int, a_int]
  , __mul__       |-> Typed (Prim "int" "__mul__") [a_int, a_int]
  , __div__       |-> Typed (Prim "int" "__div__") [a_int, a_int]
  , __divmod__    |-> Typed (Prim "int" "__divmod__") [a_int, a_int]
  , __floordiv__  |-> Typed (Prim "int" "__floordiv__") [a_int, a_int]
  , __pow__       |-> Typed (Prim "int" "__pow__") [a_int, a_int]
  , __neq__       |-> Typed (Prim "int" "__neq__") [a_int, a_int]
  , __eq__        |-> Typed (Prim "int" "__eq__") [a_int, a_int]
  , __lt__        |-> Typed (Prim "int" "__lt__") [a_int, a_int]
  , __le__        |-> Typed (Prim "int" "__le__") [a_int, a_int]
  , __gt__        |-> Typed (Prim "int" "__gt__") [a_int, a_int]
  , __ge__        |-> Typed (Prim "int" "__ge__") [a_int, a_int]
  , __str__       |-> Typed (Prim "int" "__str__") [a_int]
  ]
\end{code}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection*{Literal integers}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\rulesection{Evaluating an integer literal}
\label{fw-rewrite-IntExpr}
\begin{code}
rewrite  (State  ( heap                        ) envs  stack ( FwExpr (IntExpr i) )) =
         (state  ( heap <+> [a |-> IntValue i] ) envs  stack ( BwResult a         ))
   where  a = freeAddr heap
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection*{Integer primitives}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%if style == newstyle
\begin{code}
rewrite s = rwpart1 s
\end{code}
%endif

\rulesection{The addition primitive}
\begin{code}
rwpart1  (State  heap  envs  stack  ( FwApp  (Prim "int" "__add__") [a_x, a_y]                       )) =
         (state  heap  envs  stack  ( FwExpr (IntExpr (getIntValue heap a_x + getIntValue heap a_y)) ))
\end{code}
\label{fw-rewrite-Prim-int.__add__}

\rulesection{The equals primitive}
\begin{code}
rwpart1  (State  heap  envs  stack  ( FwApp  (Prim "int" "__eq__") [a_x, a_y]                          )) =
         (state  heap  envs  stack  ( FwExpr (BoolExpr (getIntValue heap a_x == getIntValue heap a_y)) ))
\end{code}

\rulesection{The constructor primitive}
\begin{code}
rwpart1   (State  ( heap               )  envs  stack  ( FwApp (Prim "int" "__new__") [a_c] )) =
          (state  ( heap <+> [a |-> o] )  envs  stack  ( BwResult a                         ))
  where  a =  freeAddr heap
         o =  ObjValue (new [__class__ |-> a_c]) (IntValue 0)
\end{code}
\label{fw-rewrite-Prim-int.__new__}

\rulesection{The string representation}
\begin{code}
rwpart1  (State  ( heap                                                       )  envs  stack  ( FwApp (Prim "int" "__str__") [a_o] )) =
         (state  ( heap <+> [a |-> StringValue (show (getIntValue heap a_o))] )  envs  stack  ( BwResult a                         ))
  where a = freeAddr heap
\end{code}

%if style == newcode
\begin{code}
rwpart1 (State  heap  envs  stack  ( FwApp  (Prim "int" prim) [a_x, a_y] )) = case prim of
  "__sub__" -> mkNumBinOp (-)
  "__mul__" -> mkNumBinOp (*)
  "__div__" -> mkNumBinOp div
  "__divmod__" -> mkNumBinOp mod
  "__floordiv__" -> mkNumBinOp div
  "__pow__" -> mkNumBinOp (^)
  "__neq__" -> mkCompBinOp (/=)
  "__lt__" -> mkCompBinOp (<)
  "__le__" -> mkCompBinOp (<=)
  "__gt__" -> mkCompBinOp (>)
  "__ge__" -> mkCompBinOp (>=)
  where mkNumBinOp op = state heap envs stack ( FwExpr (IntExpr (x `op` y)))
        mkCompBinOp op = state heap envs stack (FwExpr (BoolExpr (x `op` y)))
        x = getIntValue heap a_x
        y = getIntValue heap a_y
\end{code}

\begin{code}
rwpart1 _ = skipState
\end{code}
%endif

%}
