%if style == newcode
\begin{code}
module Print(rewrite) where

import VirtualMachine
import ObjectTheory
import RewriterMonad(printString)
\end{code}
%endif

%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\chapter{Print statements} \label{chapter:print}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
The print statement~|PrintStmt e| prints a string representation of |e| to the standard output. The string representation of |e| is determined by its class function~|__str__|. The program in Listing~\ref{example:print} defines an object with a special string representation.

\begin{lstlisting}[ caption={A simple Person class with a custom string representation}
                  , label=example:print ]
class Person(object) :
	def __init__(self, name, age) :
		self.name = name
		self.age = age
	def __str__(self) :
		return self.name + " is " + self.age.__str__() + " years old"

print(Person("Gideon", 25)) # prints 'Gideon is 25 years old'
\end{lstlisting}

Classes that do not define their own string representation inherit the |__str__| attribute from their superclass(es). Every object has a string representation because all classes inherit |__str__| from object. The primitive~|object.__str__| (specified in Rule~\ref{fw-rewrite-Prim-object.__str__}) uses an objects address to create a string representation that is unique for every object.


\rulesection{Evaluating the printed expression}
\label{fw-rewrite-PrintStmt}
First, the expression |e| to be printed is evaluated.

\begin{code}
rewrite  (State  heap  envs  ( stack                 )  ( FwStmt (PrintStmt e) )) =
         (state  heap  envs  ( stack :|: PrintFrame1 )  ( FwExpr e             ))
\end{code}


\rulesection{Getting a string representation of the printed value}
\label{bw-rewrite-PrintFrame1}
Next, we call the class function~|__str__| of |a| to obtain a string representation of |a|. We don't need to check whether an object has a class function~|__str__|, unlike for example the |__iter__| attribute in Rule~\ref{bw-rewrite-ForFrame1}.

\begin{code}
rewrite (State  heap  envs  ( stack :|: PrintFrame1 )  ( BwResult a      )) =
        (state  heap  envs  ( stack :|: PrintFrame2 )  ( FwApp a_f [a]   ))
  where a_f = get (getInhAttrs heap a) __str__
\end{code}


\rulesection{Printing the string}
\label{bw-rewrite-PrintFrame2-succ}
Finally, the string representation~|getStringValue heap a| of the object can be printed. Provided that the class function~|__str__| returned a string.

%{
%format >> =
%format printString str = 
%format => = "\xRightarrow{\text{print}~"heap(a)"}"
\begin{code}
rewrite                                   (State  heap  envs  ( stack :|: PrintFrame2 )  ( BwResult a           ))  | isInstOf a a_str heap  =  
  printString (getStringValue heap a) >>  (state  heap  envs  ( stack                 )  ( BwResult a_None      ))
\end{code}
%}


\rulesection{Faulty string representation}
\label{bw-rewrite-PrintFrame2-fail}
If the class function~|__str__| did not return a string, we raise a type error.

\begin{code}
rewrite  (State  heap  envs  ( stack :|: PrintFrame2 )  ( BwResult a           )) | not (isInstOf a a_str heap) =
         (state  heap  envs  ( stack                 )  ( raise a_TypeError [] ))
\end{code}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%if style==newcode
\begin{code}
rewrite _ = skipState
\end{code}
%endif
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%}
