module PythonToken where

import UU.Scanner.GenToken
import UU.Scanner.GenTokenOrd
import UU.Scanner.GenTokenSymbol

type PythonToken = GenToken String PythonTokenType String

data PythonTokenType =
  HexNumber | OctNumber | DecNumber 
  | FloatNumber | ImagNumber
  | LitAddr   
  | Name
  | LitString
  | Newline | Whitespace | Comment
  | Indent | Dedent
  deriving (Show, Eq, Ord)

instance Show (GenToken [Char] PythonTokenType [Char]) where
  show (ValToken Indent str pos) = "<indent>"
  show (ValToken Dedent str pos) = "<dedent>"
  show (ValToken Newline str pos) = "<newline>"
  show (ValToken tp str pos) = str
  show (Reserved str pos) = str

getTokenLines :: [PythonToken] -> [[PythonToken]]
getTokenLines [] = []
getTokenLines ts = getTokenLines' id ts

getTokenLines' :: ([PythonToken] -> [PythonToken]) -> [PythonToken] -> [[PythonToken]]
getTokenLines' bld [] = [bld []]
getTokenLines' bld (ValToken Newline _ _:tl) = bld [] : getTokenLines tl
getTokenLines' bld (hd:tl) = getTokenLines' (bld.(hd:)) tl
