module PythonTokenParser where

import UU.Parsing.Interface(IsParser(..))
import UU.Parsing.Derived(pListSep, pPacked)
import UU.Scanner.Position(Pos)
import UU.Scanner.GenTokenParser(pReserved, pValToken)

import PythonToken

pKeyPos           :: IsParser p PythonToken => String -> p Pos
pKeyPos  keyword  =  pReserved keyword

pOperatorPos      :: IsParser p PythonToken => String -> p Pos
pOperatorPos      =  pKeyPos

pSpecPos          :: IsParser p PythonToken => Char -> p Pos
pSpecPos s        =  pReserved [s]

pKey              :: IsParser p PythonToken => String -> p String
pKey  key         =  key <$ pKeyPos key

pOperator         :: IsParser p PythonToken => String -> p String
pOperator         = pKey

pSpec             :: IsParser p PythonToken => Char -> p String 
pSpec c           =  [c] <$ pSpecPos c
  
pLitStringPos, pOctNumberPos, pDecNumberPos, pHexNumberPos, pFloatNumberPos,
  pImagNumberPos, pNamePos, pNewlinePos, pLitAddrPos, 
  pWhitespacePos, pIndentPos, pDedentPos :: IsParser p PythonToken => p (String,Pos)

pLitAddrPos     =   pValToken LitAddr       "<address>"
pLitStringPos   =   pValToken LitString     "<string>"        
pOctNumberPos   =   pValToken OctNumber     "0"       
pDecNumberPos   =   pValToken DecNumber     "0"       
pHexNumberPos   =   pValToken HexNumber     "0"
pFloatNumberPos =   pValToken FloatNumber   "0.0"
pImagNumberPos  =   pValToken ImagNumber    "0j"
pNamePos        =   pValToken Name          "<identifier>" 
pNewlinePos     =   pValToken Newline       "<newline>"     
pIndentPos      =   pValToken Indent        "<indent>"     
pDedentPos      =   pValToken Dedent        "<dedent>"     
pWhitespacePos  =   pValToken Whitespace    "<whitespace>"

pLitString, pOctNumber, pDecNumber, pHexNumber, pFloatNumber, pImagNumber, pLitAddr,
  pName, pNewline, pWhitespace, pIndent, pDedent :: IsParser p PythonToken => p String

pLitAddr = fst <$> pLitAddrPos
pLitString = fst <$> pLitStringPos
pOctNumber = fst <$> pOctNumberPos
pDecNumber = fst <$> pDecNumberPos
pHexNumber = fst <$> pHexNumberPos
pFloatNumber = fst <$> pFloatNumberPos
pImagNumber = fst <$> pImagNumberPos
pName = fst <$> pNamePos
--pOperator = fst <$> pOperatorPos
pNewline = fst <$> pNewlinePos
pIndent = fst <$> pIndentPos
pDedent = fst <$> pDedentPos
pWhitespace = fst <$> pWhitespacePos
  
pComma, pSemi, pOParen, pCParen, pOBrack, pCBrack, pOCurly, pCCurly,
  pColon, pDot, pAt
   :: IsParser p PythonToken => p String

pComma  = pSpec ','
pDot    = pSpec '.'
pSemi   = pSpec ';'
pColon  = pSpec ':'
pAt     = pSpec '@'
pOParen = pSpec '('
pCParen = pSpec ')'
pOBrack = pSpec '['
pCBrack = pSpec ']'
pOCurly = pSpec '{'
pCCurly = pSpec '}'

--pCommaPos, pSemiPos, pOParenPos, pCParenPos, pOBrackPos, pCBrackPos, pOCurlyPos, pCCurlyPos
--   :: IsParser p PythonToken => p Pos
--
--pCommaPos  = pSpecPos ','
--pSemiPos   = pSpecPos ';'
--pOParenPos = pSpecPos '('
--pCParenPos = pSpecPos ')'
--pOBrackPos = pSpecPos '['
--pCBrackPos = pSpecPos ']'
--pOCurlyPos = pSpecPos '{'
--pCCurlyPos = pSpecPos '}'

pCommas ::  IsParser p PythonToken => p a -> p [a]
pSemics ::  IsParser p PythonToken => p a -> p [a]
pParens ::  IsParser p PythonToken => p a -> p a
pBracks ::  IsParser p PythonToken => p a -> p a
pCurly  ::  IsParser p PythonToken => p a -> p a

pCommas  = pListSep pComma
pSemics  = pListSep pSemi
pParens  = pPacked pOParen pCParen
pBracks  = pPacked pOBrack pCBrack
pCurly   = pPacked pOCurly pCCurly

pParens_pCommas :: IsParser p PythonToken => p a -> p [a]
pBracks_pCommas :: IsParser p PythonToken => p a -> p [a]
pCurly_pSemics  :: IsParser p PythonToken => p a -> p [a]

pParens_pCommas = pParens.pCommas
pBracks_pCommas = pBracks.pCommas
pCurly_pSemics  = pCurly .pSemics

