{
module PythonTokenizer where

import UU.Scanner.GenToken
import UU.Scanner.Position

import PythonToken
}

%wrapper "basic"

$digit = 0-9
$hexdigit = [0-9a-fA-F]
$alpha = [a-zA-Z_]
$alphanum = [$digit $alpha]
$imag = [jJ]
$newline = [\n \r]

@exponent = [eE][\-\+]? $digit+
@float = $digit+ \. $digit* @exponent?
@dotfloat = \.$digit+ @exponent?

@stringPrefix = [uU]?[rR]?
@escChar = \\ . | (\\$newline)
@shortStringItem = @escChar | [^ ' \\ \n]
@shortStringItemD = @escChar | [^ \" \\ \n]
@longStringItem = @escChar  | [^ ' \\]  | (' [^'])    | (''[^'])     
                  | ('$newline) | $newline
@longStringItemD = @escChar | [^ \" \\] | (\" [^ \"]) | (\"\"[^ \"])
                  | (\"$newline) | $newline

--"

tokens :-
    
  [\ \t]+       		{ \s -> ValToken Whitespace s }
  $newline		    	{ \s -> ValToken Newline s }
  \#[^\n]*              { \s -> ValToken Comment s }

  "0"[0-7]*[lL]?        { \s -> ValToken OctNumber s }
  [1-9]$digit*[lL]?     { \s -> ValToken DecNumber s }
  0[xX]$hexdigit*[lL]?  { \s -> ValToken HexNumber s }
  "$"$digit+            { \s -> ValToken LitAddr s }

  @float                { \s -> ValToken FloatNumber s }
  @dotfloat             { \s -> ValToken FloatNumber s }
  $digit+@exponent      { \s -> ValToken FloatNumber s }

  @float $imag          { \s -> ValToken ImagNumber s }
  @dotfloat $imag       { \s -> ValToken ImagNumber s }
  $digit+@exponent?$imag { \s -> ValToken ImagNumber s }
 
  @stringPrefix ''' @longStringItem* '''        { \s -> ValToken LitString s }
  @stringPrefix \"\"\" @longStringItemD* \"\"\"  { \s -> ValToken LitString s }
  @stringPrefix ' @shortStringItem* '           { \s -> ValToken LitString s }
  @stringPrefix \" @shortStringItemD* \"        { \s -> ValToken LitString s }

  "**"=? | ">>"=? | "<<"=? | "<>" | "!=" | "//"=? | "~" | "|"=?
                        { \s -> Reserved s }
  [\$\+\\\-\*\/\%&\^=\<\>]=? { \s -> Reserved s }
  [\[\]\{\}\(\)]        { \s -> Reserved s }
  [:\;\.\,`\@]          { \s -> Reserved s }

  "and" | "as" | "assert" | "break" | "class" | "continue"
  | "def" | "del" | "elif" | "else" | "except" | "exec"
  | "finally" | "for" | "from" | "global" | "if" | "import"
  | "in" | "is" | "lambda" | "not" | "or" | "pass" | "print"
  | "raise" | "return" | "try" | "while" | "with" | "yield"
       { \s -> Reserved s }

  $alpha$alphanum*  { \s -> ValToken Name s }

{

}
