%if style == newcode
\begin{code}
module Result where

import Preliminaries(intercalate)
import PythonAST
\end{code}
%endif

%format (Prim (a) (b)) = a"."b
%format (Addr a) = a
%format (Inst (a) (b)) = a.b
%format (Typed (a) (as)) = a : as

%if style == newcode
\begin{code}
data Result = Addr Int | Prim String String | Inst Result Result | Typed Result [Result]
      deriving Eq

instance Show Result where
  show (Addr i) = show i
  show (Prim obj attr) = obj ++ "." ++ attr
  show (Inst obj func) = show obj ++ show func
  show (Typed func cs) = 
    show func ++ ":: (" ++ intercalate ","(map show cs) ++ ") -> ?"
\end{code}
%endif

%format isInsted (a) = a "\in" "\iota^{\iota}"
%format isNotInsted (a) = a "\notin" "\iota^{\iota}"
%format isTyped (a) = a "\in" "\iota::\mathrm{T}"
%format isNotTyped (a) = a "\notin" "\iota::\mathrm{T}"
%format addrExpr (a) = a

%if style == newcode
\begin{code}
isInsted :: Result -> Bool
isInsted (Inst _ _) = True
isInsted (Typed a _) = isInsted a
isInsted _ = False

isNotInsted a = not (isInsted a)

isTyped a = not (isNotTyped a)

isNotTyped :: Result -> Bool
isNotTyped (Typed _ _) = False
isNotTyped _ = True

addrExpr (Addr a) = AddrExpr a
\end{code}
%endif

%format as_1
%format as_2
%format as'
%format as'_1

%format a_x
%format a_y
%format a_f
%format a_i
%format env_0

%format a_self
%format e_self
%format self = "\mathtt{self}"
%format a_other
%format other = "\mathtt{other}"

%format a_None
%format a_Type
%format a_TypeT
%format a_object
%format a_objectT
%format a_emptyTuple
%format a_str
%format a_list
%format a_tuple
%format a_dict
%format a_bool
%format a_True
%format a_False
%format a_int
%format a_function
%format a_BaseException
%format a_TypeError
%format a_NameError
%format a_NameErrorType
%format a_AttributeError
%format a_UnboundLocalError
%format a_ImportError
%format a_SyntaxError
%format a_StopIteration
%format a_ValueError
%format a_IndexError

%format None = "\mathtt{None}"
%format Type = "\mathtt{type}"
%format TypeT = "\mathtt{(type,)}"
%format object = "\mathtt{object}"
%format objectT = "\mathtt{(object,)}"
%format Module = "\mathtt{module}"
%format int = "\mathtt{int}"
%format str = "\mathtt{str}"
%format list = "\mathtt{list}"
%format dict = "\mathtt{dict}"
%format tuple = "\mathtt{tuple}"
%format emptyTuple = "\mathtt{()}"
%format bool = "\mathtt{bool}"
%format True = "\mathtt{True}"
%format False = "\mathtt{False}"
%format function = "\mathtt{function}"
%format generator = "\mathtt{generator}"
%format Number = "\mathtt{Number}"

%format BaseException = "\mathtt{BaseException}"
%format TypeError ="\mathtt{TypeError}"
%format NameError = "\mathtt{NameError}"
%format AttributeError = "\mathtt{AttributeError}"
%format SyntaxError = "\mathtt{SyntaxError}"
%format ImportError = "\mathtt{ImportError}"
%format UnboundLocalError = "\mathtt{UnboundLocalError}"
%format StopIteration = "\mathtt{StopIteration}"
%format ValueError = "\mathtt{ValueError}"
%format IndexError = "\mathtt{IndexError}"


%if style == newcode
\begin{code}
a_None = Addr 0
a_NoneType = Addr 1
a_Type = Addr 2
a_object = Addr 3
a_bool = Addr 4
a_True = Addr 5
a_False = Addr 6
a_int = Addr 7
a_list = Addr 8
a_ListIterator = Addr 9
a_file = Addr 10
a_str = Addr 11
a_dict = Addr 12
a_function = Addr 13
a_generator = Addr 14
a_tuple = Addr 15

a_ms = Addr 20
a_pwd = Addr 21
a_main = Addr 22
a_ModuleType = Addr 23
a_args = Addr 24
a_sys = Addr 25
a_os = Addr 26
a_name = Addr 27
a_stdout = Addr 28
env_0 = Addr 29
a_path = Addr 30

a_BaseException = Addr 100
a_TypeError = Addr 102
a_NameError = Addr 104
a_IndexError = Addr 106
a_SyntaxError = Addr 109
a_AttributeError = Addr 111
a_StopIteration = Addr 112
a_UnboundLocalError = Addr 114
a_ImportError = Addr 116
a_ValueError = Addr 117

a_emptyTuple = Addr 1000
a_TypeT = Addr 1002
a_objectT = Addr 1003
a_BaseExceptionT = Addr 1004
\end{code}
%endif
