module RewriterMonad
  ( RewriterM
  , startMetaState
  , debug
  , trace
  , incStepCount
  , getStepCount
  , enableTrace
  , disableTrace
  , isTracing
  , printString
  ) where

import qualified Control.Monad.State as S


type RewriterM a = S.StateT MetaState IO a

data MetaState = MetaState { stepCounter :: Int, printTrace :: Bool }

startMetaState = MetaState { stepCounter = 1, printTrace = False }

debug :: (Show a) => a -> RewriterM ()
debug = S.liftIO . print

trace :: RewriterM() -> RewriterM ()
trace action = do
  s <- S.get
  if printTrace s then action else return ()

printString :: String -> RewriterM ()
printString = S.liftIO . putStrLn

incStepCount :: RewriterM Int
incStepCount = do
  s <- S.get
  S.put (s { stepCounter = stepCounter s + 1})
  return (stepCounter s)

getStepCount :: RewriterM Int
getStepCount = S.get >>= return . stepCounter

enableTrace :: RewriterM ()
enableTrace = S.modify (\s -> s { printTrace = True })

disableTrace :: RewriterM ()
disableTrace = S.modify (\s -> s { printTrace = False })

isTracing :: RewriterM Bool
isTracing = S.get >>= return . printTrace
