%if style == newcode
\begin{code}
module Strings where
import VirtualMachine
import ObjectTheory

o_string :: Value
\end{code}
%endif
%format o_string

%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Strings}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%\begin{itemize}
%\item are immutable
%\item can be inherited
%\item implements container primitives and some number primitives
%\end{itemize}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection*{The string class}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\begin{code}
o_string = new
  [ __class__     |->  a_Type
  , __bases__     |->  a_objectT
  , __new__       |->  Prim "str" "__new__"
  , __add__       |->  Typed (Prim "str" "__add__") [a_str, a_str]
  , __mul__       |->  Typed (Prim "str" "__mul__") [a_str, a_int]
  , __len__       |->  Typed (Prim "str" "__len__") [a_str]
  , __eq__        |->  Typed (Prim "str" "__eq__") [a_str, a_str]
  , __getitem__   |->  Typed (Prim "str" "__getitem__") [a_str, a_int]
  , __str__       |->  Typed (Prim "str" "__str__") [a_str]
  , "append"      |->  Typed (Prim "str" "__add__") [a_str, a_str]
  ]
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection*{Literal strings}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\rulesection{Evaluating a string literal}
\label{fw-rewrite-StringExpr}
\begin{code}
rewrite  (State  ( heap                           )  env  stack  ( FwExpr    (StringExpr t) )) =
         (state  ( heap <+> [a |-> StringValue t] )  env  stack  ( BwResult  a              ))
  where a = freeAddr heap
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection*{String primitives}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\rulesection{The string constructor primitive}
\begin{code}
rewrite  (State  ( heap               )  env  stack  ( FwApp (Prim "str" "__new__") [a_c] )) =
         (state  ( heap <+> [a |-> o] )  env  stack  ( BwResult a                         ))
  where  a =  freeAddr heap
         o =  ObjValue (new [__class__ |-> a_c]) (StringValue [])
\end{code}


\rulesection{The string constructor primitive with starting string}
\begin{code}
rewrite  (State  ( heap               )  env  stack  ( FwApp (Prim "str" "__new__") [a_c, a_str] )) =
         (state  ( heap <+> [a |-> o] )  env  stack  ( BwResult a                                ))
  where  a =  freeAddr heap
         o =  ObjValue (new [__class__ |-> a_c]) (StringValue (getStringValue heap a_str))
\end{code}


\rulesection{The length of string primitive}
\begin{code}
rewrite  (State  heap  env  stack  ( FwApp (Prim "str" "__len__") [a]                  )) =
         (state  heap  env  stack  ( FwExpr (IntExpr (length (getStringValue heap a))) ))
\end{code}


%\rulesection{The append to string primitive}
%\begin{code}
%rewrite  (State  ( heap                      )  env  stack  ( FwApp (Prim "str" "append") [a_self, a_val] )) =
%         (state  ( heap <+> [a_self |-> lst] )  env  stack  ( BwResult a_None                             ))
%  where lst = StringValue (getStringValue heap a_self ++ (getStringValue heap a_val))
%\end{code}


\rulesection{The string addition operator primitive}
\begin{code}
rewrite  (State  ( heap               )  env  stack  ( FwApp (Prim "str" "__add__") [a_self, a_other] )) =
         (state  ( heap <+> [a |-> s] )  env  stack  ( BwResult a                                     ))
   where  s = StringValue (getStringValue heap a_self ++ getStringValue heap a_other)
          a = freeAddr heap
\end{code}

%if style=newcode
\rulesection{The string multiplication operator primitive}
\begin{code}
rewrite   (State  ( heap               )  env  stack  ( FwApp (Prim "str" "__mul__") [a_self, a_other] )) =
          (state  ( heap <+> [a |-> s] )  env  stack  ( BwResult a                                     ))
  where  s = StringValue (repeatStr (getIntValue heap a_other) (getStringValue heap a_self))
         a = freeAddr heap
\end{code}
%endif


\rulesection{The string equals primitive}
\begin{code}
rewrite  (State  heap  env  stack  ( FwApp  (Prim "str" "__eq__") [a_x, a_y]                                )) =
         (state  heap  env  stack  ( FwExpr (BoolExpr (getStringValue heap a_x == getStringValue heap a_y)) ))
\end{code}


\rulesection{The string get character at index primitive}
\begin{code}
rewrite               (State  ( heap                                )  env  stack  ( FwApp (Prim "str" "__getitem__") [a_self, a_i] ))
  | i < length s   =  (state  ( heap <+> [a |-> StringValue [s!!i]] )  env  stack  ( BwResult a                                     ))
  | otherwise      =  (state  ( heap                                )  env  stack  ( raise a_IndexError []                          ))
  where  i = getIntValue heap a_i
         s = getStringValue heap a_self
         a = freeAddr heap
\end{code}


\rulesection{The string to string conversion primitive}
\begin{code}
rewrite  (State  ( heap                                                   )  env  stack  ( FwApp (Prim "str" "__str__") [a_o] )) =
         (state  ( heap <+> [a |-> StringValue (getStringValue heap a_o)] )  env  stack  ( BwResult a                         ))
  where a = freeAddr heap
\end{code}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%if style==newcode
\begin{code}
rewrite _ = skipState
\end{code}
%endif
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%}
%if style==newcode
\begin{code}
repeatStr :: Int -> [a] -> [a]
repeatStr 0 str = []
repeatStr i str = str ++ repeatStr (i-1) str
-- repeatStr i = concat . take i . repeat
\end{code}
%endif
