\documentclass[tocleft]{scrbook}
\usepackage{a4wide}
\usepackage[fleqn]{amsmath}
\usepackage{amsfonts}
\usepackage{mathtools}
\usepackage[silent]{polytable}
\usepackage{listings}
\usepackage{url}
\usepackage{pgf}
\usepackage{color}
\usepackage{subfigure}
\usepackage{tikz}
\usepackage[pdftex,bookmarks=true]{hyperref}
\usetikzlibrary{arrows}

\hypersetup{
pdfauthor = {Gideon Smeding},
pdftitle = {An executable operational semantics for Python},
}

% rulesection
\newcounter{rule}
\numberwithin{rule}{chapter}
\let\origsubsubsection\subsubsection
%\renewcommand\subsubsection[1]{%
\newcommand\rulesection[1]{%
  \origsubsubsection{#1}%
  \refstepcounter{rule}%
  \setcounter{equation}{\value{rule}}%
  \addtocounter{equation}{-1}%
  \addcontentsline{rules}{rule}{\protect\numberline{\therule}{\ignorespaces #1}}}
% list of rules
\makeatletter
\newcommand\listofrules{%
  \chapter*{Rules}%
  \if@@tocleft\before@@starttoc{rules}\fi%
  \@@starttoc{rules}%
  \if@@tocleft\after@@starttoc{rules}\fi}
\let\l@@rule\l@@figure
\makeatother

%include polycode.fmt
%include Rule.lhs

% the results and attributes only have formatting rules, not text
%include Result.lhs
%include Attributes.lhs

\lstset{ language=Python
       , basicstyle=\ttfamily
       , columns=flexible
	   , frame=single
	   , captionpos=below
	   , float
	   }
 
\newcommand{\minpy}[0]{\textit{minpy}}
\newcommand{\comma}[0]{\text{\texttt{,}}}
\newcommand{\semicolon}[0]{\text{\texttt{;}}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%\title{An executable operational semantics for Python}
%\date{January, 2009\\
%INF/SCR-08-29}
%\author{Gideon Joachim Smeding}

\begin{document}

\begin{titlepage}
\begin{center}
{\Large Thesis for the degree of Master of Science}

\vspace{1cm}

{\Huge An executable operational semantics for Python}

\vspace{1cm}

{\huge Gideon Joachim Smeding} 

\vspace{2cm}

{\large January, 2009}

{\large \textsc{inf/scr-08-29}}

\vfill % fill vertical space

\includegraphics[width=80mm]{UU_logo_fullcolor_uncoated_sol_left.pdf}

\vspace{1cm}

\begin{minipage}{0.6\textwidth}
\begin{flushleft}
Center for Software Technology \\
Dept.\ of Information and Computing Sciences \\
Universiteit Utrecht \\
Utrecht, The Netherlands
\end{flushleft}
\end{minipage}%
%
\begin{minipage}{0.3\textwidth}
\begin{flushright}
\textit{Supervisors:} \\
dr.\ Andres L\"oh \\
prof.\ dr.\ S.D. Swierstra 
\end{flushright}
\end{minipage}

\end{center}
\end{titlepage}

\chapter*{Abstract}
Programming languages are often specified only in an informal manner; in the available documentation, the language behavior is described by examples and text. Only the implementation, a compiler or interpreter, describes the exact semantics of constructs.

Python is no different. It is described by an informal manual and a number of implementations. No systematic, formal descriptions of its semantics are available.  

We developed a formal semantics for a comprehensive subset of Python called \minpy. The semantics are described in literate Haskell. The source files are compiled to an interpreter as well as the formal specifications in this document. In a sense, this document is the documented source code of the interpreter.


\tableofcontents

%let appendix=False

%include Introduction.lhs

%include Preliminaries.lhs

%include PythonAST.lhs

%include VirtualMachine.lhs

%include ObjectTheory.lhs

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% states
%format (State (h) s1 (env) s2 (s) s3 (a) st) = "\bigl\langle" h s1 env s2 s s3 a st "\bigr\rangle"
%format (state (h) s1 (env) s2 (s) s3 (a) st) = "\bigl\langle" h s1 env s2 s s3 a st "\bigr\rangle"

%format SEP = ",\,"
%format STOP = "\vphantom{~}"
%format (CASE (x) (s) (c)) = s "\;\;" "\text{if }" c
%format (OTHE (x) (s) (c)) = s "\;\;" "\text{otherwise}"
%format (GUARD (cond)) = "\text{if }" cond
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%include BasicRewrites.phs

%include Variables.phs

%include Functions.phs

%include Objects.phs

%include Exceptions.phs

%include ControlStructures.phs

%include Print.phs

%include Operators.phs

\chapter{Dynamic code execution} \label{chapter:dynamic-code}
The exec and import statements are remarkably similar. Both dynamically parse and execute a block of code, and both statements can execute a block in an isolated environment. We will first explore the simpler exec statement, and then the import statement.
%include Exec.phs
%include Import.phs

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%\part{Conclusion}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%include Discussion.lhs

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\appendix
%let appendix=True
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\chapter{Builtin values}\label{chapter:builtins}
%include Objects.phs
%include Functions.phs
%%include Exceptions.phs
%include Numbers.phs
%include Booleans.phs
%include Strings.phs
%include Lists.phs
%include Tuples.phs
%include Dictionaries.phs

%%include Modules.phs

%%include BasicIO.phs

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%include Glossary.lhs
\listofrules
\bibliographystyle{plain}
\bibliography{references}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\end{document}
