module Tracer where

import VirtualMachine
import qualified Data.Map as M
import RewriterMonad


showStackDelta :: (Show a) => String -> Stack a -> String
showStackDelta x EmptyStack = "[]"
showStackDelta x (MarkedStack i stack) = x ++ "_" ++ show i
showStackDelta x (s :|: f) = showStackDelta x s ++ ":|:" ++ show f


showStateDelta :: Int -> State -> State -> String
showStateDelta stepno (State heap envs stack i) (State heap' envs' stack' i')
  =  "state_" ++ show stepno ++ " = "
  ++ "<"  ++ showHeapDelta stepno heap heap'
  ++ ", " ++ showStackDelta "envs" envs'
  ++ ", " ++ showStackDelta "stack" stack'
  ++ ", " ++ show i'
  ++ "> "
--  ++ "\ref{" ++ mkRuleReference stack i ++ "}"
--
--mkRuleReference (stack :|: f) (Addr a) =
--  "bw-" ++ mkStack

markState :: State -> RewriterM State
markState (State heap envs stack i) = do
  tracing <- isTracing
  if tracing 
    then do cnt <- incStepCount
            let envs' = markStack cnt envs
            let stack' = markStack cnt stack
            return (State heap envs' stack' i)
    else return (State heap envs stack i)

markStack :: Int -> Stack a -> Stack a
markStack cnt stack@(MarkedStack _ _) = stack
markStack cnt stack = MarkedStack cnt stack


showHeapDelta :: Int -> Heap -> Heap -> String
showHeapDelta i (Heap a) (Heap b) = 
  let rms = M.keys (M.difference a b)
      new = M.toAscList (M.differenceWith diff b a)
      diff x y = if x == y then Nothing else Just x
  in heapDeltaToString i rms new

heapDeltaToString :: Int -> [Int] -> [(Int, Value)] -> String
heapDeltaToString i [] [] = "heap_" ++ show (i-1)
heapDeltaToString i as [] = heapDeltaToString i [] [] 
                            ++ "<->" ++ intercalate "," (map show as)
heapDeltaToString i [] bs = heapDeltaToString i [] [] 
                            ++ "<+>" ++ intercalate "," (map showPair bs)
heapDeltaToString i as bs = parenthesis (heapDeltaToString i as [])
                            ++ "<+>" ++ intercalate "," (map showPair bs)

parenthesis :: String -> String
parenthesis str = '(' : str ++ ")"

showPair :: (Show a, Show b) => (a,b) -> String
showPair (x, y) = show x ++ " |-> " ++ show y
