%if style == newcode
\begin{code}
module Tuples(rewrite, o_tuple) where
import VirtualMachine
import ObjectTheory

o_tuple :: Value
\end{code}
%endif
%format o_tuple

%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Tuples}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%\begin{itemize}
%\item immutable shape: length cannot be changed
%\item can be derived from?
%\end{itemize}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{The tuple class}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\begin{code}
o_tuple = new
  [ __class__      |->  a_Type
  , __bases__      |->  a_objectT
  , __getitem__    |->  Typed (Prim "tuple" "__getitem__") [a_tuple, a_int]
  , __new__        |->  Prim "tuple" "__new__"
  , __len__        |->  Prim "tuple" "__len__"
  ]
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection*{Literal tuples}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\rulesection{Evaluating a tuple literal}
\label{fw-rewrite-TupleExpr}
%format firstTupleFrame (es) = ("\circ", es)
%format emptyTupleValue = (,)
\begin{code}
rewrite          (State  ( heap                             )  env  ( stack                         )  ( FwExpr (TupleExpr es) )) = case es of
  (e:es')    ->  (state  ( heap                             )  env  ( stack :|: firstTupleFrame es' )  ( FwExpr e              ))
  otherwise  ->  (state  ( heap <+> [a |-> emptyTupleValue] )  env  ( stack                         )  ( BwResult a            ))
  where a = freeAddr heap
\end{code}

\rulesection{Evaluating the elements of a tuple literal}
%format es'
\begin{code}
rewrite            (State  ( heap                                     )  env  ( stack :|: TupleFrame as es           )  ( BwResult a  )) = case es of
    (e:es')    ->  (state  ( heap                                     )  env  ( stack :|: TupleFrame (as ++ [a]) es' )  ( FwExpr e    ))
    otherwise  ->  (state  ( heap <+> [a' |-> TupleValue (as ++ [a])] )  env  ( stack                                )  ( BwResult a' ))
  where a' = freeAddr heap
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection*{Tuple primitives}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\rulesection{The tuple constructor primitive}
\begin{code}
rewrite  (State  ( heap               )  env  stack  ( FwApp (Prim "tuple" "__new__") [a_c] )) =
         (state  ( heap <+> [a |-> o] )  env  stack  ( BwResult a                           ))
   where  a =  freeAddr heap
          o =  ObjValue (new [__class__ |-> a_c]) (TupleValue [])
\end{code}


\rulesection{The length of tuple primitive}
\begin{code}
rewrite  (State  heap  env  stack  ( FwApp (Prim "tuple" "__len__") [a]               )) =
         (state  heap  env  stack  ( FwExpr (IntExpr (length (getTupleValue heap a))) ))
\end{code}


\rulesection{The get item at index primitive}
\begin{code}
rewrite              (State  heap  env  stack  ( FwApp (Prim "tuple" "__getitem__") [a_self, a_i] ))
  | i < length as =  (state  heap  env  stack  ( BwResult (as!!i)                                 ))
  | otherwise     =  (state  heap  env  stack  ( Raise a_IndexError                               ))
   where  i   = getIntValue heap a_i
          as  = getTupleValue heap a_self
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%if style==newcode
\begin{code}
rewrite _ = skipState
\end{code}
%endif
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%}
%if style==newcode
\begin{code}
firstTupleFrame es = TupleFrame [] es
emptyTupleValue = TupleValue []
\end{code}
%endif
