#!/usr/bin/perl
# This script does two more or less intertwined things:
# 1. convert haskell syntax (like guards, and case expressions) to a syntax that can be processed by lhs2TeX
# 2. Put rules into a specific format: using the tex \Rule layout

undef $/; # read whole file/input at once, not line by line

$begin = "\\begin{code}\n";
$end = "\\end{code}";

$atom = "(?:[^ (\n]+|\\( .*? \\))";
$sep ="  SEP  ";
$nep ="       ";
$stop = "  STOP";
$tileq = "(?:<=|>=|==|[^=\n])*";

while (<>) {
    # create rewrite with curly bracket from cases
    # some trickery here to insert the scrutinee (\3) into each case line:
    # the scoped formatting rule for SCR is set to the scrutinee, which causes the
    # SCR to be replaced in every CASE
    s{\\begin\{code\}\n
      ^(rewrite|rwpart.)([^\n]*?)=\ case\ ([^\n]*?)\ of\n
      (.*?)
      (\ *where.*?)?
      \\end\{code\}
     }{%\{\n%format SCR = \3\n\\Rule\{$begin               \2$end\}\{$begin\4$end\}\{$begin\5$end\}\n%\}\n}gmsx;

    # create 'single line' rewrite rule with arrow rewrite
    s{\\begin\{code\}\n
      (rewrite|rwpart.)($tileq\ *)=\s*\n
      (.*?)
      (\ *where.*?)?
      \\end\{code\}
     }{\\SimpleRule\{$begin         \2\n=>\3$end\}\{$begin\4$end\}}gmsx;
     #}{\\mathhs\n\\begin\{equation\}$begin         \2\n=>\3$end\\end\{equation\}\n\\plainhs}gmsx;

    # replace rewrites with guards by rules with curly bracket
    s{\\begin\{code\}\n
      ^(rewrite|rwpart.)([^\n]*\n)
      (\ +\|.*?)
      (\ *where.*?)?
      \\end\{code\}
     }{\\Rule\{$begin               \2$end\}\{$begin\3$end\}\{$begin\4$end\}}gmsx;

    # replace otherwise cases e.g., | otherwise  = B  to OTHE (A) (B) (A)
    s/^( +)\|( *otherwise *)= (.*)/\1OTHE (\2)    \3 (  \2)/gm;
        s/^( +)(otherwise *) -> (.*)/\1OTHE (\2 )    \3 (  \2)/gm;

    # replace cases e.g., | A  = B  to CASE (A) (B) (A)
    s/^( +)\|($tileq)= (.*)/\1CASE (\2)    \3 (  \2)/gm;
        s/^( +)(.*?) -> (.*)/\1CASE (\2 )    \3 (  SCR = \2)/gm;
    
    # replace guards in single line rule
    s/\s+\|\s+(.*)/(GUARD (\1))/g;

    # instert separations (commas)
    s/\(([sS]tate *)($atom *)($atom *)($atom *)($atom *)\)/(\1\2$sep\3$sep\4$sep\5$stop)/g;

    # remove string encapsulation in primitives
    s/(Prim *)"(.*?)"( *)"(.*?)"/\1\2 \4\3   /g;

    # replace if then else  (disabled because the TeX stuff doesn't work yet)
    #s/if\ (.*?)\ then\ (.*?)\ else\ (.*?) \n/IfThenElse (\1) (\2) (\3)\n/gmx;

    print;
}
