import sys

def max(x, y) :
    if x < y :
        return y
    else :
        return x

def min(x, y) :
    if x < y :
        return x
    else :
        return y

def len(x) :
    return x.__len__()

class TestReport(object) :
    lineLength = 77
    successMsg = "[success]"
    failMsg = "[failure]"
    msgSepSpace = 2
    finalMsgLength = max(len(successMsg), len(failMsg))

    # maximum message length, after which lines are wrapped
    maxMsgLength = lineLength - finalMsgLength - msgSepSpace

    def __init__(self) :
        # pointer position indicating the length of the current line
        self.caretPos = 0

        # false if waiting for a new test, true if waiting for a result
        self.testIsRunning = False

        # counters for no successes and failures
        self.successCnt = 0
        self.failureCnt = 0


    def reportTestStart(self, testPath) :
        #assert not self.testIsRunning
        self.testIsRunning = True
        self.testPath = testPath
        self.testDescr = ""

        self.isExpectingException = False
        self.expectedException = None

        self.ensureNewline()
        self.writeWrapped(self.testPath)


    def setDescr(self, descr) :
        self.testDescr = descr


    def reportSuccess(self) :
        #assert self.testIsRunning
        self.testIsRunning = False

        self.writeRight(self.successMsg)
        self.ensureNewline()

        self.successCnt = self.successCnt + 1


    def reportFailure(self, exception) :
        #assert self.testIsRunning
        self.testIsRunning = False

        self.writeRight(self.failMsg)
        self.ensureNewline()
    
        self.writeWrapped(self.testDescr)
        self.ensureNewline()
        #self.writeWrapped(str(exception))
        print(exception)
        
        self.ensureNewline()
        
        self.failureCnt = self.failureCnt + 1


    def writeRight(self, msg) :
        # fill with spaces until we're at the right position
        noSpaces = self.lineLength - len(msg) - self.caretPos
        sys.stdout.write(" " *  noSpaces)
        sys.stdout.write(msg)

        sys.stdout.write("\n")
        self.caretPos = 0


    def expectException(self, e) :
        self.isExpectingException = True
        self.expectedException = e


    def writeWrapped(self, msg) :
        #for n in range(0, len(msg), self.maxMsgLength) :
        n = 0
        while n < msg.__len__() :
            # a wrapped message has to be put on a new line, so break
            if 0 < n :
                self.ensureNewline()

            # print the maximum possible piece of the message
            #sys.stdout.write(msg[n:self.maxMsgLength])
            i = n
            while i < min(len(msg), self.maxMsgLength) :
                sys.stdout.write(msg[i])
                i = i + 1

            # update the caret position with the added stuff
            self.caretPos = self.caretPos + min(len(msg), self.maxMsgLength)

            n = n + self.maxMsgLength


    def ensureNewline(self) :
        if 0 < self.caretPos :
            sys.stdout.write("\n")
            self.caretPos = 0
